# -*- coding: utf-8 -*-
"""
@File    :   DynConfig.py
@Time    :   2023/07/13 14:20:02
@Author  :   Polyisoprene 
@Version :   1.0
@Desc    :   None
"""
import json
from os import getcwd, makedirs, path
from typing import Optional
from zipfile import ZipFile

from loguru import logger

try:
    import skia
except ImportError as e:
    logger.error(e)
    logger.warning(
        "Missing dependent files \n\n please install dependence: \n\n ---------------------------------------\n\n "
        "Ubuntu: apt install libgl1-mesa-glx \n\n ArchLinux: pacman -S libgl \n\n Centos: yum install mesa-libGL -y "
        "\n\n---------------------------------------")
from .DynStyle import PolyStyle


class MakeStaticFile:
    def __init__(self, data_path: Optional[str] = None) -> None:
        self.data_path: str = data_path

    @property
    def check_cache_file(self) -> None:
        """查询缓存文件是否存在"""
        # 查询是否有data_path参数
        # 没有的话静态文件目录就设置在程序的运行目录
        # 当前文件所在的目录的路径
        current_dir = path.dirname(path.abspath(__file__))
        if self.data_path is None:
            # 确定程序的运行目录的路径
            program_running_path = getcwd()
            # 静态文件的路径
            static_path = path.join(program_running_path, "Static")
            # 如果不存在静态目录将自带的压缩文件解压过去
            if not path.exists(static_path):
                logger.info("未检测到static目录")
                self.unzip_file(
                    "用户未传入data路径,将在程序运行目录创建static目录",
                    "创建static目录中...",
                    current_dir,
                    program_running_path
                )
                logger.info("static目录创建成功")
        else:
            # 设置静态文件的目录
            static_path = path.join(self.data_path, "Static")
            if not path.exists(self.data_path):
                # 如果data_path不存在创建这个目录
                makedirs(self.data_path)
            if not path.exists(static_path):
                self.unzip_file(
                    "未检测到static目录",
                    "使用用户传入路径创建static目录中...",
                    current_dir,
                    self.data_path
                )
                logger.info("static目录创建成功")
        font_cache_path = path.join(static_path, "font_family.json")
        if not path.exists(font_cache_path):
            logger.info("创建系统安装的所有字体的名称列表文件")
            font_list = list(skia.FontMgr())
            with open(font_cache_path, "w") as f:
                f.write(json.dumps(font_list, ensure_ascii=False))
            logger.info("字体列表文件创建完成")
            logger.info(f"文件存储于：{font_cache_path}")
        else:
            new_font_list = list(skia.FontMgr())
            with open(font_cache_path, "w+") as f:
                if file_content := f.read():
                    old_font_list = json.loads(file_content)
                    if new_font_list != old_font_list:
                        f.write(json.dumps(new_font_list, ensure_ascii=False))
                else:
                    f.write(json.dumps(new_font_list, ensure_ascii=False))

        return static_path

    def unzip_file(self, arg0, arg1, src_path, target_path):
        logger.info(arg0)
        logger.info(arg1)
        file = ZipFile(path.join(src_path, "Static.zip"))
        file.extractall(target_path)


class SetDynStyle:
    def __init__(self, font_family: str, emoji_font_family: str, font_style: str) -> None:
        self.font_family = font_family
        self.font_style = font_style
        self.emoji_font_family = emoji_font_family

    @property
    def set_style(self) -> PolyStyle:
        cfg_obj = {
            "color": {
                "font_color": {
                    "text": (0, 0, 0, 255),
                    "sub_title": (153, 162, 170, 255),
                    "title": (0, 0, 0, 255),
                    "name_big_vip": (251, 107, 148, 255),
                    "name_small_vip": (60, 232, 78, 255),
                    "rich_text": (0, 161, 214, 255),
                    "white": (255, 255, 255, 255)
                },
                "background": {
                    "normal": (255, 255, 255, 255),
                    "repost": (241, 242, 243, 255),
                    "border": (229, 233, 239, 255)
                }

            },
            "font": {
                "font_family": self.font_family,
                "emoji_font_family": self.emoji_font_family,
                "font_style": self.get_font_style(),
                "font_size": {
                    "name": 45,
                    "text": 40,
                    "time": 35,
                    "title": 30,
                    "sub_title": 20
                }
            }
        }

        return PolyStyle(**cfg_obj)

    def get_font_style(self):
        style_map = {"Normal": skia.FontStyle().Normal(), "Bold": skia.FontStyle().Bold(
        ), "Italic": skia.FontStyle().Italic(), "BoldItalic": skia.FontStyle().BoldItalic()}
        return style_map.get(self.font_style, skia.FontStyle().Normal())

# if __name__ == "__main__":
#     a = SetDynStyle("Noto Sans CJK SC","Normal").set_style
#     print(a.color.backgroud.repost)



# -*- coding: utf-8 -*-
'''
@File    :   DynStyle.py
@Time    :   2023/07/16 20:42:46
@Author  :   Polyisoprene
@Version :   1.0
@Desc    :   None
'''

from typing import Any

from pydantic import BaseModel


class FontSize(BaseModel):
    text: int
    name: int
    time: int
    title: int
    sub_title: int


class FontColor(BaseModel):
    text: tuple
    title: tuple
    name_big_vip: tuple
    name_small_vip: tuple
    rich_text: tuple
    sub_title: tuple
    white: tuple


class BackgroundColor(BaseModel):
    normal: tuple
    repost: tuple
    border: tuple


class FontCfg(BaseModel):
    font_family: str
    emoji_font_family: str
    font_style: Any
    font_size: FontSize


class ColorCfg(BaseModel):
    font_color: FontColor
    background: BackgroundColor


class PolyStyle(BaseModel):
    font: FontCfg
    color: ColorCfg



# -*- coding: utf-8 -*-
# @Time    : 2023/7/15 下午8:12
# @Author  : Polyisoprene
# @File    : DynText.py

import asyncio
from os import path
from typing import Optional

import emoji
import numpy as np
import skia
from dynamicadaptor.Content import Text
from loguru import logger

from .DynStyle import PolyStyle
from .DynTools import paste, merge_pictures, get_pictures


class BiliText:
    """渲染动态的文字部分"""

    def __init__(self, static_path: str, style: PolyStyle) -> None:
        self.emoji_path = path.join(static_path, "Cache", "Emoji")
        self.src_path = path.join(static_path, "Src")
        self.style = style
        surface = skia.Surface(1080, 60)
        self.canvas = surface.getCanvas()
        self.bg_color = None
        self.offset = 40
        self.x_bound = 1030
        self.image_list = []
        self.emoji_dict = {}

    async def run(self, dyn_text: Text, repost: bool = False) -> Optional[np.ndarray]:
        self.text_font = skia.Font(skia.Typeface.MakeFromName(self.style.font.font_family,self.style.font.font_style),self.style.font.font_size.text)
        self.emoji_font = skia.Font(skia.Typeface.MakeFromName(self.style.font.emoji_font_family,self.style.font.font_style),self.style.font.font_size.text)
        self.bg_color = self.style.color.background.repost if repost else self.style.color.background.normal
        self.canvas.clear(skia.Color(*self.bg_color))
        try:
            tasks = []
            if dyn_text.topic is not None:
                tasks.append(self.make_topic(dyn_text.topic.name))
            if dyn_text.text:
                tasks.append(self.make_text_image(dyn_text))
            await asyncio.gather(*tasks)
            return await merge_pictures(self.image_list)
        except Exception as e:
            logger.exception("Error")
            return None

    async def make_text_image(self, dyn_text):
        emoji_list = []
        emoji_name_list = []
        rich_list = []
        for i in dyn_text.rich_text_nodes:
            if i.type == "RICH_TEXT_NODE_TYPE_EMOJI":
                if i.text not in emoji_name_list:
                    emoji_name_list.append(i.text)
                    emoji_list.append(i.emoji.icon_url)
            elif i.type != "RICH_TEXT_NODE_TYPE_TEXT":
                rich_list.append(i)
        result = await asyncio.gather(self.get_emoji(emoji_list, emoji_name_list), self.get_rich_pic(rich_list))
        await self.draw_text(result[1], dyn_text)
        if self.offset != 40:
            self.image_list.append(self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))

    async def get_emoji(self, emoji_url: list, emoji_name: list):
        emoji_pic = []
        emoji_index = []
        emoji_url_list = []
        temp = {}
        icon_size = int(self.style.font.font_size.text * 1.5)
        for i, emoji_text in enumerate(emoji_name):
            emoji_path = path.join(self.emoji_path, f"{emoji_text}.png")
            if path.exists(emoji_path):
                emoji_pic.append(skia.Image.open(emoji_path))
            else:
                emoji_url_list.append(emoji_url[i])
                emoji_index.append(i)
        if emoji_url_list:
            result = await get_pictures(emoji_url_list, (icon_size, icon_size))
            for i, j in enumerate(emoji_index):
                emoji_path = path.join(self.emoji_path, f"{emoji_name[j]}.png")
                emoji_pic.insert(j, result[i])
                if result[i] is not None:
                    result[i].save(emoji_path)
        for i, j in enumerate(emoji_name):
            temp[j] = emoji_pic[i]
        self.emoji_dict = temp

    async def get_emoji_text(self, text: str):
        result = emoji.emoji_list(text)
        temp = {}
        for i in result:
            temp[i["match_start"]] = [i["match_end"], i["emoji"]]
        return temp

    async def get_rich_pic(self, rich_list):
        rich_dic = {}
        rich_size = self.style.font.font_size.text
        for i in rich_list:
            if i.type == "RICH_TEXT_NODE_TYPE_VOTE":
                if "vote" not in rich_dic:
                    img_path = path.join(self.src_path, "vote.png")
                    img = skia.Image.open(img_path).resize(rich_size, rich_size)
                    rich_dic["vote"] = img
            elif i.type == "RICH_TEXT_NODE_TYPE_LOTTERY":
                if "lottery" not in rich_dic:
                    img_path = path.join(self.src_path, "lottery.png")
                    img = skia.Image.open(img_path).resize(rich_size, rich_size)
                    rich_dic["lottery"] = img
            elif i.type == "RICH_TEXT_NODE_TYPE_GOODS":
                if "goods" not in rich_dic:
                    img_path = path.join(self.src_path, "taobao.png")
                    img = skia.Image.open(img_path).resize(rich_size, rich_size)
                    rich_dic["goods"] = img
            elif i.type in {"RICH_TEXT_NODE_TYPE_WEB", "RICH_TEXT_NODE_TYPE_BV"}:
                if "link" not in rich_dic:
                    img_path = path.join(self.src_path, "link.png")
                    img = skia.Image.open(img_path).resize(rich_size, rich_size)
                    rich_dic["link"] = img
            elif i.type == "RICH_TEXT_NODE_TYPE_CV":
                if "cv" not in rich_dic:
                    img_path = path.join(self.src_path, "article.png")
                    img = skia.Image.open(img_path).resize(rich_size, rich_size)
                    rich_dic["cv"] = img
            else:
                if "link" not in rich_dic:
                    img_path = path.join(self.src_path, "link.png")
                    img = skia.Image.open(img_path).resize(rich_size, rich_size)
                    rich_dic["link"] = img
        return rich_dic

    async def make_topic(self, topic: str) -> None:
        """
        make topic image
        @param topic: topic text
        @return: None
        """
        topic_size = self.style.font.font_size.text
        topic_img = skia.Image.open(path.join(self.src_path, "new_topic.png")).resize(topic_size, topic_size)
        icon_size = int(topic_size * 1.5)
        surface = skia.Surface(1080, icon_size + 10)
        canvas = surface.getCanvas()
        canvas.clear(skia.Color(*self.bg_color))
        await paste(canvas, topic_img, (45, 15))

        paint = skia.Paint(AntiAlias=True, Color=skia.Color(*self.style.color.font_color.rich_text))
        font_name = None
        offset = 45 + topic_size + 10
        font = None
        for i in topic:
            if typeface := skia.FontMgr().matchFamilyStyleCharacter(
                    self.style.font.font_family,
                    self.style.font.font_style,
                    ["zh", "en"],
                    ord(i),
            ):
                text_family_name = typeface.getFamilyName()
                if font_name != text_family_name:
                    font_name = text_family_name
                    font = skia.Font(typeface, topic_size)
            else:
                font = skia.Font(None, topic_size)
            blob = skia.TextBlob(i, font)
            canvas.drawTextBlob(blob, offset, 50, paint)
            offset += font.measureText(i)
        self.image_list.append(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))

    async def draw_text(self, rich_list: list, dyn_text: Text):
        self.canvas.clear(skia.Color(*self.bg_color))
        for i in dyn_text.rich_text_nodes:
            if i.type in {"RICH_TEXT_NODE_TYPE_AT", "RICH_TEXT_NODE_TYPE_TEXT", "RICH_TEXT_NODE_TYPE_TOPIC"}:
                if i.type in {"RICH_TEXT_NODE_TYPE_AT", "RICH_TEXT_NODE_TYPE_TOPIC"}:
                    color = skia.Color(*self.style.color.font_color.rich_text)
                else:
                    color = skia.Color(*self.style.color.font_color.text)
                await self.draw_plain_text(i.text, color)
            elif i.type == "RICH_TEXT_NODE_TYPE_EMOJI":
                await self.draw_emoji(i.text)
            else:
                await self.draw_rich_text(i, rich_list)

    async def draw_plain_text(self, dyn_detail, color):
        font = None
        dyn_detail = dyn_detail.translate(str.maketrans({'\r': ''}))
        paint = skia.Paint(AntiAlias=True, Color=color)
        emoji_info = await self.get_emoji_text(dyn_detail)
        total = len(dyn_detail) - 1
        offset = 0
        while offset <= total:
            j = dyn_detail[offset]
            if j == "\n":
                self.canvas.saveLayer()
                self.canvas.clear(skia.Color(*self.bg_color))
                self.image_list.append(self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
                self.canvas.restore()
                offset += 1
                self.offset = 40
                continue
            if offset in emoji_info.keys():
                j = emoji_info[offset][1]
                offset = emoji_info[offset][0]
                font = self.emoji_font
            else:
                offset +=1
                font = self.text_font
            if font.textToGlyphs(j)[0] == 0:
                if typeface := skia.FontMgr().matchFamilyStyleCharacter(
                    self.style.font.font_family,
                    self.style.font.font_style,
                    ["zh", "en"],
                    ord(j[0]),
                ):
                    font = skia.Font(typeface, self.style.font.font_size.text)
                else:
                    font = self.text_font
            measure = font.measureText(j)
            blob = skia.TextBlob(j, font)
            self.canvas.drawTextBlob(blob, self.offset, 50, paint)
            self.offset += measure
            if self.offset > self.x_bound:
                self.offset = 40
                self.canvas.saveLayer()
                self.canvas.clear(skia.Color(*self.bg_color))
                self.image_list.append(self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
                self.canvas.restore()

    async def draw_emoji(self,emoji_detail):
        img = self.emoji_dict[emoji_detail]
        if img is not None:
            img_size = img.dimensions().width()
            await paste(self.canvas,img, (int(self.offset),0))
            self.offset += img_size+5
            if self.offset >= self.x_bound:
                self.canvas.saveLayer()
                self.canvas.clear(skia.Color(*self.bg_color))
                self.image_list.append(self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
                self.canvas.restore()
                self.offset = 40

    async def draw_rich_text(self, text_detail, rich_list):
        if text_detail.type == "RICH_TEXT_NODE_TYPE_VOTE":
            icon = rich_list["vote"]
        elif text_detail.type == "RICH_TEXT_NODE_TYPE_LOTTERY":
            icon = rich_list["lottery"]
        elif text_detail.type == "RICH_TEXT_NODE_TYPE_GOODS":
            icon = rich_list["goods"]
        elif text_detail.type == "RICH_TEXT_NODE_TYPE_CV":
            icon = rich_list["cv"]
        else:
            icon = rich_list["link"]
        await paste(self.canvas,icon,(int(self.offset),int(60-icon.dimensions().height())/2))
        self.offset += icon.dimensions().width()+5
        if self.offset >= self.x_bound:
            self.canvas.saveLayer()
            self.canvas.clear(skia.Color(*self.bg_color))
            self.image_list.append(self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
            self.canvas.restore()
            self.offset = 40
        paint = skia.Paint(AntiAlias=True, Color=skia.Color(*self.style.color.font_color.rich_text))
        font_name = None
        font = None
        for i in text_detail.text:
            if typeface := skia.FontMgr().matchFamilyStyleCharacter(
                    self.style.font.font_family,
                    self.style.font.font_style,
                    ["zh", "en"],
                    ord(i),
            ):
                text_family_name = typeface.getFamilyName()
                if font_name != text_family_name:
                    font_name = text_family_name
                    font = skia.Font(typeface, self.style.font.font_size.text)
            else:
                font = skia.Font(None, self.style.font.font_size.text)
            blob = skia.TextBlob(i, font)
            self.canvas.drawTextBlob(blob, self.offset, 50, paint)
            self.offset += font.measureText(i)
            if self.offset >= self.x_bound:
                self.canvas.saveLayer()
                self.canvas.clear(skia.Color(*self.bg_color))
                self.image_list.append(self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
                self.canvas.restore()
                self.offset = 40


import re
from abc import ABC, abstractmethod
from os import path
from typing import Optional

import numpy as np
import skia
from dynamicadaptor.AddonCard import Additional
from loguru import logger

from .DynConfig import PolyStyle
from .DynTools import DrawText, get_pictures, paste


class AbstractAdditional(ABC):
    def __init__(self, src_path: str, style: PolyStyle, additional: Additional) -> None:
        self.style = style
        self.additional = additional
        self.src_path = src_path
        self.canvas = None
        self.text_font = skia.Font(skia.Typeface.MakeFromName(self.style.font.font_family, self.style.font.font_style),
                                   self.style.font.font_size.text)
        self.emoji_font = skia.Font(
            skia.Typeface.MakeFromName(
                self.style.font.emoji_font_family, self.style.font.font_style),
            self.style.font.font_size.text)

    @abstractmethod
    async def run(self, repost: bool) -> Optional[np.ndarray]:
        pass

    async def make_badge(self, badge: str, font_size: int, pos: tuple, img_size: tuple, text_pos: tuple):
        self.text_font.setSize(font_size)
        surface = skia.Surface(*img_size)
        canvas = surface.getCanvas()
        canvas.clear(skia.Color(*self.style.color.font_color.name_big_vip))
        blob = skia.TextBlob(badge, self.text_font)
        paint = skia.Paint(AntiAlias=True, Color=skia.Color4f.kWhite)
        canvas.drawTextBlob(blob, text_pos[0], text_pos[1], paint)
        tag_img = skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                       colorType=skia.ColorType.kRGBA_8888_ColorType)
        tag_img = await self.make_round_cornor(tag_img, 10)
        await paste(self.canvas, tag_img, pos)

    async def make_round_cornor(self, img, corner: int):
        surface = skia.Surface(img.width(), img.height())
        mask = surface.getCanvas()
        paint = skia.Paint(
            Style=skia.Paint.kFill_Style,
            Color=skia.Color(255, 255, 255, 255),
            AntiAlias=True)
        rect = skia.Rect.MakeXYWH(0, 0, img.width(), img.height())
        mask.drawRoundRect(rect, corner, corner, paint)
        image_array = np.bitwise_and(img.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                     mask.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
        return skia.Image.fromarray(image_array, colorType=skia.ColorType.kRGBA_8888_ColorType)

    async def draw_shadow(self, canvas, pos: tuple, corner: int, bg_color):
        x, y, width, height = pos
        rec = skia.Rect.MakeXYWH(x, y, width, height)
        paint = skia.Paint(
            Color=skia.Color(*bg_color),
            AntiAlias=True,
            ImageFilter=skia.ImageFilters.DropShadow(
                0, 0, 10, 10, skia.Color(120, 120, 120))
        )
        if corner != 0:
            canvas.drawRoundRect(rec, corner, corner, paint)
        else:
            canvas.drawRect(rec, paint)


class BiliAdditional:
    def __init__(self, static_path: str, style: PolyStyle) -> None:
        self.src_path = path.join(static_path, "Src")
        self.style = style

    async def run(self, additional: Additional, repost: bool = False) -> Optional[np.ndarray]:
        additional_type = additional.type
        try:
            if additional_type == "ADDITIONAL_TYPE_RESERVE":
                return await DynAddReserve(self.src_path, self.style, additional).run(repost)
            elif additional_type == "ADDITIONAL_TYPE_UPOWER_LOTTERY":
                return await DynAddUpOwerLottery(self.src_path, self.style, additional).run(repost)
            elif additional_type == "ADDITIONAL_TYPE_GOODS":
                return await DynAddGoods(self.src_path, self.style, additional).run(repost)
            elif additional_type == "ADDITIONAL_TYPE_UGC":
                return await DynAddUgc(self.src_path, self.style, additional).run(repost)
            elif additional_type == "ADDITIONAL_TYPE_VOTE":
                return await DynAddVote(self.src_path, self.style, additional).run(repost)
            elif additional_type == "ADDITIONAL_TYPE_COMMON":
                return await DynAddCommon(self.src_path, self.style, additional).run(repost)
            else:
                logger.warning(f"{additional_type} IS NOT SUPPORT NOW")
                return None
        except Exception as e:
            logger.exception(e)
            return None


class DynAddReserve(AbstractAdditional):

    async def run(self, repost) -> Optional[np.ndarray]:
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 225)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 20, 1010, 185), 15, background_color)
            await self.make_desc()
            await self.make_badge("预约", self.style.font.font_size.text, (850, 75), (170, 75), (45, 50))
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_desc(self):
        draw = DrawText(self.style)
        if self.additional.reserve.desc3 is not None:
            await draw.draw_text(self.canvas, self.additional.reserve.title, self.style.font.font_size.time,
                                 (75, 70, 740, 70, 0), self.style.color.font_color.text)
            desc_1 = f"{self.additional.reserve.desc1.text}  {self.additional.reserve.desc2.text}"
            await draw.draw_text(self.canvas, desc_1, self.style.font.font_size.title,
                                 (75, 120, 810, 120, 0), self.style.color.font_color.sub_title)

            await draw.draw_text(self.canvas, self.additional.reserve.desc3.text, self.style.font.font_size.title,
                                 (105, 170, 810, 170, 0), self.style.color.font_color.rich_text)

            lottery_img = skia.Image.open(
                path.join(self.src_path, "lottery.png")).resize(40, 40)
            await paste(self.canvas, lottery_img, (65, 138))
        else:
            await draw.draw_text(self.canvas, self.additional.reserve.title, self.style.font.font_size.time,
                                 (75, 100, 740, 100, 0), self.style.color.font_color.text)

            desc_1 = f"{self.additional.reserve.desc1.text}  {self.additional.reserve.desc2.text}"
            await draw.draw_text(self.canvas, desc_1, self.style.font.font_size.title,
                                 (75, 160, 810, 160, 0), self.style.color.font_color.sub_title)


class DynAddUpOwerLottery(AbstractAdditional):

    async def run(self, repost) -> Optional[np.ndarray]:
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 225)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 20, 1010, 185), 15, background_color)
            await self.make_desc()
            await self.make_badge("去看看", self.style.font.font_size.time, (860, 75), (155, 75), (25, 50))
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_desc(self):
        draw = DrawText(self.style)
        await draw.draw_text(self.canvas, self.additional.upower_lottery.title, self.style.font.font_size.time,
                             (75, 100, 740, 100, 0), self.style.color.font_color.text)

        await draw.draw_text(self.canvas, self.additional.upower_lottery.desc.text, self.style.font.font_size.title,
                             (105, 160, 810, 160, 0), self.style.color.font_color.rich_text)
        lottery_img = skia.Image.open(
            path.join(self.src_path, "lottery.png")).resize(40, 40)
        await paste(self.canvas, lottery_img, (65, 128))


class DynAddGoods(AbstractAdditional):

    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 310)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 50, 1010, 240), 15, background_color)
            await self.make_cover()
            await self.make_title_desc()
            await DrawText(self.style).draw_text(self.canvas, self.additional.goods.head_text,
                                                 self.style.font.font_size.sub_title, (
                                                     45, 30, 1010, 80, 0),
                                                 self.style.color.font_color.sub_title)
            await self.make_badge("去看看", self.style.font.font_size.time, (860, 125), (155, 75), (25, 50))
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_cover(self):
        url_list = []
        for i in self.additional.goods.items:
            url = re.sub("@(\d+)h_(\d+)w\S+", "", i.cover)
            url_list.append(f"{url}@160w_160h_1c.webp")
        covers = await get_pictures(url_list, (190, 190))
        if len(covers) > 1:
            for i, j in enumerate(covers):
                x = 45 + i * 200
                if x > 1000:
                    break
                await paste(self.canvas, await self.make_round_cornor(j, 10), (x, 75))
        else:
            await paste(self.canvas, await self.make_round_cornor(covers[0], 10), (60, 75))

    async def make_title_desc(self):
        if len(self.additional.goods.items) > 1:
            return
        draw = DrawText(self.style)
        await draw.draw_text(self.canvas, self.additional.goods.items[0].name, self.style.font.font_size.title,
                             (275, 140, 800, 140, 0), self.style.color.font_color.text)

        price = f"{self.additional.goods.items[0].price}起"
        await draw.draw_text(self.canvas, price, self.style.font.font_size.title,
                             (295, 210, 800, 210, 0), self.style.color.font_color.rich_text)


class DynAddUgc(AbstractAdditional):
    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 280)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 20, 1010, 240), 15, background_color)
            rec = skia.Rect.MakeXYWH(35, 20, 1010, 240)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.make_cover()
            await self.make_title_desc()
            await self.make_sub_tag()
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_cover(self):
        cover = await get_pictures(f"{self.additional.ugc.cover}@340w_195h_1c.webp")
        await paste(self.canvas, await self.make_round_cornor(cover, 10), (60, 45))

    async def make_title_desc(self):
        draw = DrawText(self.style)
        await draw.draw_text(self.canvas, self.additional.ugc.title, self.style.font.font_size.title,
                             (430, 90, 990, 140, int(self.style.font.font_size.time * 1.3)),
                             self.style.color.font_color.text)
        await draw.draw_text(self.canvas, self.additional.ugc.desc_second, self.style.font.font_size.title,
                             (430, 220, 950, 220, 0), self.style.color.font_color.sub_title)

    async def make_sub_tag(self):
        self.text_font.setSize(self.style.font.font_size.sub_title)
        size = self.text_font.measureText(self.additional.ugc.duration)
        surface = skia.Surface(int(size + 20), int(self.text_font.getSize() + 20))
        canvas = surface.getCanvas()
        canvas.clear(skia.Color(0, 0, 0, 150))
        blob = skia.TextBlob(self.additional.ugc.duration, self.text_font)
        paint = skia.Paint(AntiAlias=True, Color=skia.Color4f.kWhite)
        canvas.drawTextBlob(blob, 10, int(self.text_font.getSize() + 5), paint)
        sub_tag_img = skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                           colorType=skia.ColorType.kRGBA_8888_ColorType)
        await paste(self.canvas, await self.make_round_cornor(sub_tag_img, 10), (400 - sub_tag_img.width() - 15, 190))


class DynAddVote(AbstractAdditional):

    async def run(self, repost) -> Optional[np.ndarray]:
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 280)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 20, 1010, 240), 15, background_color)
            rec = skia.Rect.MakeXYWH(35, 20, 1010, 240)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.make_cover()
            await self.make_title_desc()
            await self.make_badge("投票", self.style.font.font_size.time, (860, 95), (155, 75), (42, 50))

            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_cover(self):
        cover = skia.Image.open(fp=path.join(self.src_path, "vote_icon.png")).resize(195, 195)
        await paste(self.canvas, await self.make_round_cornor(cover, 10), (60, 45))

    async def make_title_desc(self):
        draw = DrawText(self.style)
        await draw.draw_text(self.canvas, self.additional.vote.desc, self.style.font.font_size.text,
                             (280, 110, 810, 110, 0),
                             self.style.color.font_color.text)
        if self.additional.vote.join_num is None:
            join_num = "0人参与"
        else:
            join_num = f"{self.additional.vote.join_num}人参与"

        await draw.draw_text(self.canvas, join_num, self.style.font.font_size.time,
                             (280, 190, 810, 190, 0), self.style.color.font_color.sub_title)


class DynAddCommon(AbstractAdditional):

    async def run(self, repost) -> Optional[np.ndarray]:
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 340)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 80, 1010, 245), 15, background_color)

            await DrawText(self.style).draw_text(self.canvas, self.additional.common.head_text,
                                                 self.style.font.font_size.title, (50, 50, 1010, 90, 0),
                                                 self.style.color.font_color.sub_title)

            rec = skia.Rect.MakeXYWH(35, 80, 1010, 240)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)

            await self.make_cover()
            await self.select_badge()
            await self.select_badge()
            await self.make_title()
            await self.make_desc()

            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_cover(self):
        if self.additional.common.sub_type in {'decoration', 'game'}:
            cover_url = f"{self.additional.common.cover}@190w_190h_1c.webp"
            cover = await get_pictures(cover_url, (190, 190))
        else:
            cover_url = f"{self.additional.common.cover}@145w_195h_1c.webp"
            cover = await get_pictures(cover_url, (145, 195))
        await paste(self.canvas, await self.make_round_cornor(cover, 15), (60, 110))

    async def make_title(self):
        if self.additional.common.desc2:
            y = 150
        else:
            y = 180
        if self.additional.common.sub_type in {'decoration', 'game'}:
            x = 280
        else:
            x = 250
        await DrawText(self.style).draw_text(self.canvas, self.additional.common.title, self.style.font.font_size.text,
                                             (x, y, 780, y, 0), self.style.color.font_color.text)

    async def make_desc(self):
        draw = DrawText(self.style)
        if self.additional.common.sub_type in {'decoration', 'game'}:
            x = 280
        else:
            x = 250
        if self.additional.common.desc2:
            await draw.draw_text(self.canvas, self.additional.common.desc1, self.style.font.font_size.title,
                                 (x, 220, 780, 160, 0), self.style.color.font_color.sub_title)

            await draw.draw_text(self.canvas, self.additional.common.desc2, self.style.font.font_size.title,
                                 (x, 285, 780, 225, 0), self.style.color.font_color.sub_title)
        else:
            await draw.draw_text(self.canvas, self.additional.common.desc1, self.style.font.font_size.title,
                                 (x, 250, 780, 190, 0), self.style.color.font_color.sub_title)

    async def select_badge(self):
        badge_text_map = {
            "pugv": "去试看",
            "ogv": "去看看",
            "manga": "去追漫",
            "decoration": "去看看",
            "game": "进入"
        }
        badge_text = badge_text_map.get(self.additional.common.sub_type, "去看看")
        size = self.text_font.measureText(badge_text)
        x = int((155 - size) / 2)
        await self.make_badge(badge_text, self.style.font.font_size.time, (860, 165), (155, 75), (x, 50))



# -*- coding: utf-8 -*-
'''
@File    :   Core.py
@Time    :   2023/07/13 15:21:46
@Author  :   Polyisoprene 
@Version :   1.0
@Desc    :   None
'''
import asyncio

from dynamicadaptor.Message import RenderMessage

from .DynAdditional import BiliAdditional
from .DynConfig import MakeStaticFile, SetDynStyle
from .DynHeader import BiliHeader, Footer
from .DynMajor import BiliMajor
from .DynRepost import BiliRepost
from .DynText import BiliText
from .DynTools import merge_pictures


class DynRender:
    def __init__(self, font_family: str = "Noto Sans CJK SC",
                 emoji_font_family: str = "Noto Color Emoji",
                 font_style: str = "Normal",
                 static_path: str = None) -> None:
        """create static file and set font family and font style

        Args:
            font_family (str, optional): font family name like "Noto Sans CJK SC". Defaults to "Noto Sans CJK SC".
            emoji_font_family (str, optional):emoji font family name like "Noto Color Emoji". Defaults to "Noto Sans CJK SC".
            font_style (str, optional): font style like "Normal、Bold、Italic、BoldItalic". Defaults to "Normal".
            static_path (str, optional): static file path,must be absolute path. Defaults to None.
        """
        self.static_path = MakeStaticFile(static_path).check_cache_file
        self.style = SetDynStyle(font_family, emoji_font_family, font_style).set_style

    async def run(self, message: RenderMessage):
        tasks = [BiliHeader(self.static_path, self.style).run(message.header)]
        if message.text is not None:
            tasks.append(BiliText(self.static_path, self.style).run(message.text))
        if message.major is not None:
            tasks.append(BiliMajor(self.static_path, self.style).run(message.major))

        if message.forward is not None:
            tasks.append(BiliRepost(self.static_path, self.style).run(message.forward))

        if message.additional is not None:
            tasks.append(BiliAdditional(self.static_path, self.style).run(message.additional))

        tasks.append(Footer(self.static_path, self.style).run())
        result = await asyncio.gather(*tasks)
        return await merge_pictures(result)



# -*- coding: utf-8 -*-
# @Time    : 2023/7/15 下午9:22
# @Author  : Polyisoprene
# @File    : DynTools.py
import asyncio
from typing import List, Optional, Union

import emoji
import httpx
import numpy as np
import skia
from loguru import logger
from numpy import ndarray

from .DynStyle import PolyStyle


async def get_pictures(url: Union[str, list], size: Optional[tuple] = None):
    async with httpx.AsyncClient() as client:
        if isinstance(url, list):
            return await asyncio.gather(*[request_img(client, i, size) for i in url])
        elif isinstance(url, str):
            return await request_img(client, url, size)


async def request_img(client, url, size):
    try:
        resp = await client.get(url)
        assert resp.status_code == 200
        img = skia.Image.MakeFromEncoded(resp.content)
        if size is not None and img is not None:
            return img.resize(*size)
        return img

    except:
        logger.exception("e")
        return None

        #     pas


async def merge_pictures(img_list: List[ndarray]) -> ndarray:
    img_top = np.zeros([0, 1080, 4], np.uint8)
    if len(img_list) == 1 and img_list[0] is not None:
        return img_list[0]
    for i in img_list:
        if i is not None:
            img_top = np.vstack((img_top, i))
    return img_top


async def paste(src, target, position: tuple) -> None:
    try:
        if target is not None:
            x, y = position
            img_height = target.dimensions().fHeight
            img_width = target.dimensions().fWidth
            rec = skia.Rect.MakeXYWH(x, y, img_width, img_height)
            src.drawImageRect(target, skia.Rect(0, 0, img_width, img_height), rec)
    except Exception as e:
        logger.exception(e)


class DrawText:
    def __init__(self, style: PolyStyle):
        self.style = style
        self.text_font = skia.Font(skia.Typeface.MakeFromName(self.style.font.font_family, self.style.font.font_style),
                                   self.style.font.font_size.text)
        self.emoji_font = skia.Font(
            skia.Typeface.MakeFromName(self.style.font.emoji_font_family, self.style.font.font_style),
            self.style.font.font_size.text)

    async def draw_text(self, canvas, text: str, font_size, pos: tuple, font_color: tuple):
        paint = skia.Paint(AntiAlias=True, Color=skia.Color(*font_color))
        self.text_font.setSize(font_size)
        self.emoji_font.setSize(font_size)
        text = text.replace("\t", "")
        emoji_info = await self.get_emoji_text(text)
        total = len(text) - 1
        x, y, x_bound, y_bound, y_int = pos
        offset = 0
        while offset <= total:
            j = text[offset]
            if j == "\n":
                break
            if offset in emoji_info.keys():
                j = emoji_info[offset][1]
                offset = emoji_info[offset][0]
                font = self.emoji_font
            else:
                offset += 1
                font = self.text_font
            if font.textToGlyphs(j)[0] == 0:
                if typeface := skia.FontMgr().matchFamilyStyleCharacter(
                        self.style.font.font_family,
                        self.style.font.font_style,
                        ["zh", "en"],
                        ord(j[0]),
                ):
                    font = skia.Font(typeface, font_size)
                else:
                    font = self.text_font
            measure = font.measureText(j)
            blob = skia.TextBlob(j, font)
            canvas.drawTextBlob(blob, x, y, paint)
            x += measure
            if x > x_bound:
                y += y_int
                if y >= y_bound:
                    blob = skia.TextBlob("...", font)
                    canvas.drawTextBlob(blob, x, y - y_int, paint)
                    break
                x = pos[0]

    async def get_emoji_text(self, text):
        result = emoji.emoji_list(text)
        temp = {}
        for i in result:
            temp[i["match_start"]] = [i["match_end"], i["emoji"]]
        return temp



# -*- coding: utf-8 -*-
'''
@File    :   DynRepost.py
@Time    :   2023/07/19 15:33:23
@Author  :   Polyisoprene 
@Version :   1.0
@Desc    :   None
'''
import asyncio

from dynamicadaptor.Repost import Forward

from .DynAdditional import BiliAdditional
from .DynHeader import RepostHeader
from .DynMajor import BiliMajor
from .DynStyle import PolyStyle
from .DynText import BiliText
from .DynTools import merge_pictures


class BiliRepost:
    def __init__(self, static_path: str, style: PolyStyle) -> None:
        self.static_path = static_path
        self.style = style

    async def run(self, message: Forward):
        tasks = [RepostHeader(self.static_path, self.style).run(message.header)]
        if message.text is not None:
            tasks.append(BiliText(self.static_path, self.style).run(message.text, repost=True))
        if message.major is not None:
            tasks.append(BiliMajor(self.static_path, self.style).run(message.major, True))
            if message.additional is not None:
                tasks.append(BiliAdditional(self.static_path, self.style).run(message.additional, True))
        result = await asyncio.gather(*tasks)
        return await merge_pictures(result)



import asyncio
from os import path
from time import localtime, strftime, time
from typing import Optional

import numpy as np
import skia
from dynamicadaptor.Header import Head
from .DynConfig import logger
from .DynStyle import PolyStyle
from .DynTools import get_pictures, paste
from .DynTools import DrawText


class BiliHeader:
    """渲染动态的头部"""

    def __init__(self, static_path: str, style: PolyStyle) -> None:
        self.face_path = path.join(static_path, "Cache", "Face")
        self.pendant_path = path.join(static_path, "Cache", "Pendant")
        self.src_path = path.join(static_path, "Src")
        self.style = style
        self.canvas = None
        self.message = None

    async def run(self, header_message: Head) -> Optional[np.ndarray]:
        try:
            self.message = header_message
            surface = skia.Surface(1080, 400)
            self.canvas = surface.getCanvas()
            self.canvas.clear(skia.Color(*self.style.color.background.normal))
            result = await asyncio.gather(self.paste_logo(),
                                          self.draw_name(),
                                          self.draw_pub_time(),
                                          self.get_face_and_pendant(True),
                                          self.get_face_and_pendant()
                                          )
            await self.past_face()
            await self.paste_pendant(result[4])
            await self.paste_vip()
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception:
            logger.exception("Error")
            return None

    async def paste_pendant(self, pendant):
        """
        paste pendant onto canvas
        @param pendant:
        @return:
        """
        if pendant is not None:
            pendant = pendant.resize(190, 190)
            await self.paste(pendant, (10, 210))

    async def paste_vip(self):
        if self.message.official_verify and self.message.official_verify.type != -1:
            if self.message.official_verify.type == 0:
                img_path = path.join(self.src_path, "official_yellow.png")
            else:
                img_path = path.join(self.src_path, "official_blue.png")
            img = skia.Image.open(img_path).resize(45, 45)
            await self.paste(img, (120, 330))
        elif self.message.vip and self.message.vip.status == 1:
            if self.message.vip.avatar_subscript == 1:
                img_path = path.join(self.src_path, "big_vip.png")
            else:
                img_path = path.join(self.src_path, "small_vip.png")
            img = skia.Image.open(img_path).resize(45, 45)
            await self.paste(img, (120, 330))

    async def past_face(self):
        face = await self.get_face_and_pendant(True)
        if face:
            face = await self.circle_face(face, 120)
            await self.paste(face, (45, 245))

    async def get_face_and_pendant(self, img_type: bool = False):

        if img_type:
            img_name = f"{self.message.mid}.webp"
            img_url = f"{self.message.face}@240w_240h_1c_1s.webp"
            img_path = path.join(self.face_path, img_name)
        elif self.message.pendant and self.message.pendant.image:

            img_name = f"{self.message.pendant.pid}.png"
            img_url = f"{self.message.pendant.image}@360w_360h.webp"
            img_path = path.join(self.pendant_path, img_name)
        else:
            return None
        if path.exists(img_path):
            if time() - int(path.getmtime(img_path)) <= 43200:
                return skia.Image.open(img_path)
        img = await get_pictures(img_url)
        if img is not None:
            img.save(img_path)
            return img
        return None

    async def circle_face(self, img, size):
        surface = skia.Surface(img.dimensions().width(),
                               img.dimensions().height())
        mask = surface.getCanvas()
        paint = skia.Paint(
            Style=skia.Paint.kFill_Style,
            Color=skia.Color(255, 255, 255, 255),
            AntiAlias=True)
        radius = int(img.dimensions().width() / 2)
        mask.drawCircle(radius, radius, radius, paint)

        paint = skia.Paint(
            Style=skia.Paint.kStroke_Style,
            StrokeWidth=5,
            Color=skia.Color(251, 114, 153, 255),
            AntiAlias=True)

        image_array = np.bitwise_and(img.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                     mask.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
        canvas = skia.Canvas(image_array, colorType=skia.ColorType.kRGBA_8888_ColorType)
        canvas.drawCircle(radius, radius, radius - 2, paint)
        return skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                    colorType=skia.ColorType.kRGBA_8888_ColorType).resize(size, size)

    async def draw_pub_time(self):
        if self.message.pub_ts:
            pub_time = strftime("%Y-%m-%d %H:%M:%S",
                                localtime(self.message.pub_ts))
        elif self.message.pub_time:
            pub_time = self.message.pub_time
        else:
            pub_time = strftime("%Y-%m-%d %H:%M:%S", localtime(time()))

        await DrawText(self.style).draw_text(self.canvas, pub_time, self.style.font.font_size.time,
                                             (200, 350, 1010, 350, 0,), self.style.color.font_color.sub_title)

    async def paste_logo(self) -> None:
        logo = skia.Image.open(path.join(self.src_path, "bilibili.png")).resize(231, 105)
        await self.paste(logo, (433, 20))

    async def draw_name(self):
        # 如果是大会员的话
        if self.message.vip and self.message.vip.status == 1:
            # 如果是大会员名字是粉色
            if self.message.vip.avatar_subscript == 1:
                color = self.style.color.font_color.name_big_vip
            else:
                # 到了愚人节大会员名字会变成绿色
                color = self.style.color.font_color.name_small_vip
        else:
            color = self.style.color.font_color.text

        await DrawText(self.style).draw_text(self.canvas, self.message.name, self.style.font.font_size.name,
                                             (200, 300, 1010, 300, 0), color)

    async def paste(self, image, position: tuple) -> None:
        x, y = position
        img_height = image.dimensions().fHeight
        img_width = image.dimensions().fWidth
        rec = skia.Rect.MakeXYWH(x, y, img_width, img_height)
        self.canvas.drawImageRect(image, skia.Rect(
            0, 0, img_width, img_height), rec)


class RepostHeader:
    def __init__(self, static_path: str, style: PolyStyle) -> None:
        self.style = style
        self.static_path = static_path

    async def run(self, message: Head) -> Optional[np.ndarray]:
        surface = skia.Surface(1080, 100)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*self.style.color.background.repost))
        try:
            if message.name is None:
                return None
            if message.face is not None:
                pos = 140
                await self.draw_face(message.face, message.mid)
            else:
                pos = 35
            await self.draw_name(message.name, pos)
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def draw_face(self, url, mid):
        img = await self.get_face(mid, url)
        if img is not None:
            face = await self.circle_face(img, 80)
        await paste(self.canvas, face, (40, 10))

    async def draw_name(self, name, pos: int):
        await DrawText(self.style).draw_text(self.canvas, name, self.style.font.font_size.name, (pos, 70, 1010, 70, 0),
                                             self.style.color.font_color.rich_text)

    async def circle_face(self, img, size):
        surface = skia.Surface(img.dimensions().width(),
                               img.dimensions().height())
        mask = surface.getCanvas()
        paint = skia.Paint(
            Style=skia.Paint.kFill_Style,
            Color=skia.Color(255, 255, 255, 255),
            AntiAlias=True)
        radius = int(img.dimensions().width() / 2)
        mask.drawCircle(radius, radius, radius, paint)

        paint = skia.Paint(
            Style=skia.Paint.kStroke_Style,
            StrokeWidth=5,
            Color=skia.Color(251, 114, 153, 255),
            AntiAlias=True)

        image_array = np.bitwise_and(img.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                     mask.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
        canvas = skia.Canvas(image_array, colorType=skia.ColorType.kRGBA_8888_ColorType)
        canvas.drawCircle(radius, radius, radius - 2, paint)
        return skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                    colorType=skia.ColorType.kRGBA_8888_ColorType).resize(size, size)

    async def get_face(self, mid, url):
        img_name = f"{mid}.webp"
        img_url = f"{url}@240w_240h_1c_1s.webp"
        img_path = path.join(self.static_path, "Cache", "Face", img_name)
        if path.exists(img_path):
            if time() - int(path.getmtime(img_path)) <= 43200:
                return skia.Image.open(img_path)
        img = await get_pictures(img_url)
        if img is not None:
            img.save(img_path)
        return img


class Footer:
    def __init__(self, static_path: str, style: PolyStyle) -> None:
        self.src_path = path.join(static_path, "Src")
        self.style = style
        self.canvas = None

    async def run(self) -> Optional[np.ndarray]:
        surface = skia.Surface(1080, 110)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*self.style.color.background.normal))
        try:

            now = strftime("%Y-%m-%d %H:%M:%S", localtime(time()))
            render_time = f"图片生成于：{now}"

            await DrawText(self.style).draw_text(self.canvas, render_time, self.style.font.font_size.title,
                                                 (35, 70, 1010, 70, 0), self.style.color.font_color.sub_title)
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def draw_shadow(self, canvas, pos: tuple, corner: int, bg_color):
        x, y, width, height = pos
        rec = skia.Rect.MakeXYWH(x, y, width, height)
        paint = skia.Paint(
            Color=skia.Color(*bg_color),
            AntiAlias=True,
            ImageFilter=skia.ImageFilters.DropShadow(0, 0, 10, 10, skia.Color(120, 120, 120))
        )
        if corner != 0:
            canvas.drawRoundRect(rec, corner, corner, paint)
        else:
            canvas.drawRect(rec, paint)



# -*- coding: utf-8 -*-
'''
@File    :   DynMajor.py
@Time    :   2023/07/16 20:21:46
@Author  :   Polyisoprene
@Version :   1.0
@Desc    :   None
'''
from os import path

import skia
import numpy as np
import emoji
from typing import Optional
from loguru import logger
from .DynStyle import PolyStyle
from .DynTools import paste, get_pictures
from dynamicadaptor.Majors import Major
from math import ceil

from abc import ABC, abstractmethod


class AbstractMajor(ABC):
    def __init__(self, src_path: str, style: PolyStyle, dyn_major: Major) -> None:
        self.style = style
        self.major = dyn_major
        self.text_font = skia.Font(skia.Typeface.MakeFromName(self.style.font.font_family, self.style.font.font_style),
                                   self.style.font.font_size.text)
        self.emoji_font = skia.Font(
            skia.Typeface.MakeFromName(self.style.font.emoji_font_family, self.style.font.font_style),
            self.style.font.font_size.text)
        self.src_path = src_path
        self.canvas = None

    @abstractmethod
    async def run(self, repost) -> Optional[np.ndarray]:
        pass

    async def draw_text(self, canvas, text: str, font_size, pos: tuple, font_color: tuple):
        paint = skia.Paint(AntiAlias=True, Color=skia.Color(*font_color))
        self.text_font.setSize(font_size)
        self.emoji_font.setSize(font_size)
        text = text.replace("\t", "")
        emoji_info = await self.get_emoji_text(text)
        total = len(text) - 1
        x, y, x_bound, y_bound, y_int = pos
        offset = 0
        while offset <= total:
            j = text[offset]
            if j == "\n":
                break
            if offset in emoji_info.keys():
                j = emoji_info[offset][1]
                offset = emoji_info[offset][0]
                font = self.emoji_font
            else:
                offset += 1
                font = self.text_font
            if font.textToGlyphs(j)[0] == 0:
                if typeface := skia.FontMgr().matchFamilyStyleCharacter(
                        self.style.font.font_family,
                        self.style.font.font_style,
                        ["zh", "en"],
                        ord(j[0]),
                ):
                    font = skia.Font(typeface, font_size)
                else:
                    font = self.text_font
            measure = font.measureText(j)
            blob = skia.TextBlob(j, font)
            canvas.drawTextBlob(blob, x, y, paint)
            x += measure
            if x > x_bound:
                y += y_int
                if y >= y_bound:
                    blob = skia.TextBlob("...", font)
                    canvas.drawTextBlob(blob, x, y - y_int, paint)
                    break
                x = pos[0]

    async def get_emoji_text(self, text):
        result = emoji.emoji_list(text)
        temp = {}
        for i in result:
            temp[i["match_start"]] = [i["match_end"], i["emoji"]]
        return temp

    async def draw_shadow(self, canvas, pos: tuple, corner: int, bg_color):
        x, y, width, height = pos
        rec = skia.Rect.MakeXYWH(x, y, width, height)
        paint = skia.Paint(
            Color=skia.Color(*bg_color),
            AntiAlias=True,
            ImageFilter=skia.ImageFilters.DropShadow(0, 0, 10, 10, skia.Color(120, 120, 120))
        )
        if corner != 0:
            canvas.drawRoundRect(rec, corner, corner, paint)
        else:
            canvas.drawRect(rec, paint)

    async def make_round_cornor(self, img, corner: int):
        surface = skia.Surface(img.width(), img.height())
        mask = surface.getCanvas()
        paint = skia.Paint(
            Style=skia.Paint.kFill_Style,
            Color=skia.Color(255, 255, 255, 255),
            AntiAlias=True)
        rect = skia.Rect.MakeXYWH(0, 0, img.width(), img.height())
        mask.drawRoundRect(rect, corner, corner, paint)
        image_array = np.bitwise_and(img.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                     mask.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType))
        return skia.Image.fromarray(image_array, colorType=skia.ColorType.kRGBA_8888_ColorType)

    async def make_tag(self, tag: str, font_size: int):
        self.text_font.setSize(font_size)
        size = self.text_font.measureText(tag)
        surface = skia.Surface(int(size + 20), int(self.text_font.getSize() + 20))
        canvas = surface.getCanvas()
        canvas.clear(skia.Color(*self.style.color.font_color.name_big_vip))
        blob = skia.TextBlob(tag, self.text_font)
        paint = skia.Paint(AntiAlias=True, Color=skia.Color4f.kWhite)
        canvas.drawTextBlob(blob, 10, int(self.text_font.getSize() + 5), paint)
        tag_img = skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                       colorType=skia.ColorType.kRGBA_8888_ColorType)
        tag_img = await self.make_round_cornor(tag_img, 10)
        await paste(self.canvas, tag_img, (1010 - tag_img.width(), 50))

    async def make_sub_tag(self, sub_tag: str, font_size: int):
        self.text_font.setSize(font_size)
        size = self.text_font.measureText(sub_tag)
        surface = skia.Surface(int(size + 20), int(self.text_font.getSize() + 20))
        canvas = surface.getCanvas()
        canvas.clear(skia.Color(0, 0, 0, 150))
        blob = skia.TextBlob(sub_tag, self.text_font)
        paint = skia.Paint(AntiAlias=True, Color=skia.Color4f.kWhite)
        canvas.drawTextBlob(blob, 10, int(self.text_font.getSize() + 5), paint)
        sub_title_img = skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                             colorType=skia.ColorType.kRGBA_8888_ColorType)
        await paste(self.canvas, await self.make_round_cornor(sub_title_img, 10), (80, 525))


class BiliMajor:
    def __init__(self, static_path: str, style: PolyStyle) -> None:
        self.src_path = path.join(static_path, "Src")
        self.style = style

    async def run(self, dyn_major: Major, repost: bool = False) -> Optional[np.ndarray]:
        try:
            major_type = dyn_major.type
            if major_type == "MAJOR_TYPE_DRAW":
                return await DynMajorDraw(self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_ARCHIVE":
                return await DynMajorArchive(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_LIVE_RCMD":
                return await DynMajorLiveRcmd(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_OPUS":
                return await DynMajorArticle(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_COMMON":
                return await DynMajorCommon(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_MUSIC":
                return await DynMajorMusic(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_PGC":
                return await DynMajorPgc(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_MEDIALIST":
                return await DynMajorMediaList(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_COURSES":
                return await DynMajorCourses(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_UGC_SEASON":
                return await DynMajorUgc(self.src_path, self.style, dyn_major).run(repost)
            elif major_type == "MAJOR_TYPE_LIVE":
                return await DynMajorLive(self.src_path, self.style, dyn_major).run(repost)
            else:
                logger.warning(f"{major_type} is not supported")
                return None
        except Exception as e:
            logger.exception(e)
            return None


class DynMajorDraw:
    def __init__(self, style, major) -> None:
        self.style = style
        self.major = major

    async def run(self, repost: bool) -> Optional[np.ndarray]:
        """
        make image of major draw
        @param repost: bool
        @return:
        """
        try:
            item_count = len(self.major.draw.items)
            background_color = self.style.color.background.repost if repost else self.style.color.background.normal
            if item_count == 1:
                return await self.single_img(background_color, self.major.draw.items)
            elif item_count in {2, 4}:
                return await self.dual_img(background_color, self.major.draw.items)
            else:
                return await self.triplex_img(background_color, self.major.draw.items)
        except Exception:
            logger.exception("Error")
            return None

    async def single_img(self, background_color: str, items) -> np.ndarray:
        src = items[0].src
        img_height = items[0].height
        img_width = items[0].width
        if img_height / img_width > 4:
            img_url = f"{src}@{600}w_{800}h_!header.webp"
        else:
            img_url = src
        img = await get_pictures(img_url)
        if img is not None:
            img = img.resize(width=1008, height=int(img.height() * 1008 / img.width()))
            surface = skia.Surface(1080, img.height() + 20)
            canvas = surface.getCanvas()
            canvas.clear(skia.Color(*background_color))
            await paste(canvas, img, (36, 10))
            return canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)

    async def dual_img(self, background_color: str, items):
        url_list = []
        for item in items:
            src = item.src
            img_height = item.height
            img_width = item.width
            if img_height / img_width > 3:
                url_list.append(f"{src}@520w_520h_!header.webp")
            else:
                url_list.append(f"{src}@520w_520h_1e_1c.webp")
        imgs = await get_pictures(url_list, (520, 520))
        num = len(url_list) / 2
        back_size = int(num * 520 + 20 * num)
        surface = skia.Surface(1080, back_size)
        canvas = surface.getCanvas()
        canvas.clear(skia.Color(*background_color))

        x, y = 15, 10
        for i in imgs:
            if i is not None:
                await paste(canvas, i, (x, y))
            x += 530
            if x > 1000:
                x = 15
                y += 530
        return canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)

    async def triplex_img(self, background_color: str, items):
        url_list = []
        for item in items:
            src = item.src
            img_height = item.height
            img_width = item.width
            if img_height / img_width > 3:
                url_list.append(f"{src}@260w_260h_!header.webp")
            else:
                url_list.append(f"{src}@260w_260h_1e_1c.webp")
        num = ceil(len(items) / 3)

        imgs = await get_pictures(url_list, (346, 346))

        back_size = int(num * 346 + 20 * num)
        surface = skia.Surface(1080, back_size)
        canvas = surface.getCanvas()
        canvas.clear(skia.Color(*background_color))
        x, y = 11, 10
        for img in imgs:
            if img is not None:
                await paste(canvas, img, (x, y))
            x += 356
            if x > 1000:
                x = 11
                y += 356
        return canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)


class DynMajorArchive(AbstractMajor):

    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 695)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        tv = skia.Image.open(path.join(self.src_path, "tv.png")).resize(130, 130)
        try:
            cover = await get_pictures(f"{self.major.archive.cover}@505w_285h_1c.webp", (1010, 570))

            await self.draw_shadow(self.canvas, (35, 25, 1010, 655), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 25, 1010, 665)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.draw_text(self.canvas, self.major.archive.title, self.style.font.font_size.text,
                                 (60, 650, 980, 600, 10), self.style.color.font_color.text)
            await paste(self.canvas, cover, (35, 25))
            await paste(self.canvas, tv, (905, 455))
            if self.major.archive.badge is not None and self.major.archive.badge.text != '':
                await self.make_tag(self.major.archive.badge.text, self.style.font.font_size.text)
            await self.make_sub_tag(self.major.archive.duration_text, self.style.font.font_size.title)
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception:
            logger.exception("Error")
            return None


class DynMajorLiveRcmd(AbstractMajor):

    async def run(self, repost) -> Optional[np.ndarray]:
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 695)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            cover = await get_pictures(f"{self.major.live_rcmd.content.live_play_info.cover}@505w_285h_1c.webp",
                                       (1010, 570))

            await self.draw_shadow(self.canvas, (35, 25, 1010, 655), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 25, 1010, 665)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)

            await self.draw_text(self.canvas, self.major.live_rcmd.content.live_play_info.title,
                                 self.style.font.font_size.text, (60, 650, 980, 600, 10),
                                 self.style.color.font_color.text)
            await paste(self.canvas, cover, (35, 25))
            await self.make_tag("直播中", self.style.font.font_size.text)
            await self.make_sub_tag(self.major.live_rcmd.content.live_play_info.watched_show.text_large,
                                    self.style.font.font_size.title)
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception:
            logger.exception("Error")
            return None


class DynMajorArticle(AbstractMajor):

    async def run(self, repost) -> Optional[np.ndarray]:
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 640)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 20, 1010, 600), 20, background_color)

            rec = skia.Rect.MakeXYWH(35, 20, 1010, 600)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.draw_title_and_desc()
            await self.make_cover()
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_cover(self):
        if len(self.major.opus.pics) > 1:
            url_list = [f"{i.url}@360w_360h_1c" for i in self.major.opus.pics]
            imgs = await get_pictures(url_list, (330, 330))
            for i, j in enumerate(imgs):
                await paste(self.canvas, j, (35 + i * 340, 20))
        else:
            img = await get_pictures(f"{self.major.opus.pics[0].url}@647w_150h_1c.webp", (1010, 300))
            await paste(self.canvas, img, (35, 20))

    async def draw_title_and_desc(self):
        title = self.major.opus.title
        if len(self.major.opus.pics) > 1:
            await self.draw_text(self.canvas, title, self.style.font.font_size.text, (50, 410, 960, 330, 0),
                                 self.style.color.font_color.text)
        else:
            await self.draw_text(self.canvas, title, self.style.font.font_size.text, (50, 390, 960, 330, 0),
                                 self.style.color.font_color.text)
        desc = self.major.opus.summary.text
        await self.draw_text(self.canvas, desc, self.style.font.font_size.title,
                             (65, 460, 980, 620, int(self.style.font.font_size.title * 1.8)),
                             self.style.color.font_color.sub_title)


class DynMajorCommon(AbstractMajor):
    async def run(self, repost) -> Optional[np.ndarray]:
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 285)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 20, 1010, 245), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 20, 1010, 245)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            cover = await get_pictures(f"{self.major.common.cover}@245w_245h_1c.webp", (245, 245))
            await paste(self.canvas, cover, (35, 20))
            await self.make_title()
            await self.make_common_tag()
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_title(self):
        title = self.major.common.title
        await self.draw_text(self.canvas, title, self.style.font.font_size.text, (310, 120, 950, 120, 0),
                             self.style.color.font_color.text)
        sub_title = self.major.common.desc
        await  self.draw_text(self.canvas, sub_title, self.style.font.font_size.title, (310, 190, 970, 190, 0),
                              self.style.color.font_color.sub_title)

    async def make_common_tag(self):
        if self.major.common.badge is not None and self.major.common.badge.text != "":
            self.text_font.setSize(self.style.font.font_size.sub_title)
            size = self.text_font.measureText(self.major.common.badge.text)
            tag_width = int(size + 20)
            surface = skia.Surface(tag_width, int(self.text_font.getSize() + 20))
            canvas = surface.getCanvas()
            canvas.clear(skia.Color(*self.style.color.font_color.name_big_vip))
            blob = skia.TextBlob(self.major.common.badge.text, self.text_font)
            paint = skia.Paint(AntiAlias=True, Color=skia.Color4f.kWhite)
            canvas.drawTextBlob(blob, 10, int(self.text_font.getSize() + 5), paint)
            tag_img = skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType),
                                           colorType=skia.ColorType.kRGBA_8888_ColorType)
            await paste(self.canvas, await self.make_round_cornor(tag_img, 10), (280 - tag_width - 20, 40))


class DynMajorMusic(AbstractMajor):
    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 285)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            await self.draw_shadow(self.canvas, (35, 20, 1010, 245), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 20, 1010, 245)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            cover = await get_pictures(f"{self.major.music.cover}@245w_245h_1c.webp", (245, 245))
            await paste(self.canvas, cover, (35, 20))
            await self.make_title()
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None

    async def make_title(self):
        title = self.major.music.title
        await self.draw_text(self.canvas, title, self.style.font.font_size.text, (310, 120, 950, 120, 0),
                             self.style.color.font_color.text)
        sub_title = self.major.music.label
        await self.draw_text(self.canvas, sub_title, self.style.font.font_size.title, (310, 190, 970, 190, 0),
                             self.style.color.font_color.sub_title)


class DynMajorPgc(AbstractMajor):

    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 695)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        tv = skia.Image.open(path.join(self.src_path, "tv.png")).resize(130, 130)
        try:
            cover = await get_pictures(f"{self.major.pgc.cover}@505w_285h_1c.webp", (1010, 570))
            await self.draw_shadow(self.canvas, (35, 25, 1010, 655), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 25, 1010, 665)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.draw_text(self.canvas, self.major.pgc.title, self.style.font.font_size.text,
                                 (60, 650, 980, 600, 10), self.style.color.font_color.text)
            await paste(self.canvas, cover, (35, 25))
            await paste(self.canvas, tv, (905, 455))
            if self.major.pgc.badge is not None and self.major.pgc.badge.text != '':
                tag = self.major.pgc.badge.text
            else:
                tag = "投稿视频"
            await self.make_tag(tag, self.style.font.font_size.text)
            await self.make_sub_tag(f"{self.major.pgc.stat.play}播放", self.style.font.font_size.title)
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception:
            logger.exception("Error")
            return None


class DynMajorMediaList(AbstractMajor):

    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 695)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        tv = skia.Image.open(path.join(self.src_path, "tv.png")).resize(130, 130)
        try:
            cover = await get_pictures(f"{self.major.medialist.cover}@505w_285h_1c.webp", (1010, 570))
            await self.draw_shadow(self.canvas, (35, 25, 1010, 655), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 25, 1010, 665)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.draw_text(self.canvas, self.major.medialist.title, self.style.font.font_size.text,
                                 (60, 650, 980, 600, 10), self.style.color.font_color.text)
            await paste(self.canvas, cover, (35, 25))
            if self.major.medialist.badge is not None:
                tag = self.major.medialist.badge.text
            else:
                tag = "投稿视频"
            await self.make_tag(tag, self.style.font.font_size.text)
            await self.make_sub_tag(self.major.medialist.sub_title, self.style.font.font_size.title)
            await paste(self.canvas, tv, (905, 455))
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None


class DynMajorCourses(AbstractMajor):

    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 695)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        tv = skia.Image.open(path.join(self.src_path, "tv.png")).resize(130, 130)
        try:
            cover = await get_pictures(f"{self.major.courses.cover}@505w_285h_1c.webp", (1010, 570))
            await self.draw_shadow(self.canvas, (35, 25, 1010, 655), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 25, 1010, 665)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.draw_text(self.canvas, self.major.courses.title, self.style.font.font_size.text,
                                 (60, 650, 980, 600, 10), self.style.color.font_color.text)
            await paste(self.canvas, cover, (35, 25))
            if self.major.courses.badge is not None and self.major.courses.badge.text != '':
                tag = self.major.courses.badge.text
            else:
                tag = "投稿视频"
            await self.make_tag(tag, self.style.font.font_size.text)
            await self.make_sub_tag(self.major.courses.desc, self.style.font.font_size.title)
            await paste(self.canvas, tv, (905, 455))
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None


class DynMajorUgc(AbstractMajor):

    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 695)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        tv = skia.Image.open(path.join(self.src_path, "tv.png")).resize(130, 130)
        try:
            cover = await get_pictures(f"{self.major.ugc_season.cover}@505w_285h_1c.webp", (1010, 570))
            await self.draw_shadow(self.canvas, (35, 25, 1010, 655), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 25, 1010, 665)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.draw_text(self.canvas, self.major.ugc_season.title, self.style.font.font_size.text,
                                 (60, 650, 980, 600, 10), self.style.color.font_color.text)
            await paste(self.canvas, cover, (35, 25))
            if self.major.ugc_season.badge is not None and self.major.ugc_season.badge.text != '':
                tag = self.major.ugc_season.badge.text
            else:
                tag = "投稿视频"
            await self.make_tag(tag, self.style.font.font_size.text)
            await self.make_sub_tag(self.major.ugc_season.duration_text, self.style.font.font_size.title)
            await paste(self.canvas, tv, (905, 455))
            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None


class DynMajorLive(AbstractMajor):

    async def run(self, repost):
        background_color = self.style.color.background.repost if repost else self.style.color.background.normal
        surface = skia.Surface(1080, 695)
        self.canvas = surface.getCanvas()
        self.canvas.clear(skia.Color(*background_color))
        try:
            cover = await get_pictures(f"{self.major.live.cover}@505w_285h_1c.webp", (1010, 570))
            await self.draw_shadow(self.canvas, (35, 25, 1010, 655), 20, background_color)
            rec = skia.Rect.MakeXYWH(35, 25, 1010, 665)
            self.canvas.clipRRect(skia.RRect(rec, 20, 20), skia.ClipOp.kIntersect)
            await self.draw_text(self.canvas, self.major.live.title, self.style.font.font_size.text,
                                 (60, 650, 980, 600, 10), self.style.color.font_color.text)
            await paste(self.canvas, cover, (35, 25))
            if self.major.live.badge is not None and self.major.live.badge.text != '':
                tag = self.major.live.badge.text
            else:
                tag = "投稿视频"
            await self.make_tag(tag, self.style.font.font_size.text)

            await self.make_sub_tag(self.major.live.desc_second, self.style.font.font_size.title)

            return self.canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType)
        except Exception as e:
            logger.exception(e)
            return None
